@@ -7,6 +7,7 @@ from collections import Counter
 import dataclasses
 import datetime
 import functools
+import itertools
 import logging
 import random
 from typing import (
@@ -17,14 +18,16 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Sequence,
     Tuple,
     Type,
     TypeVar,
     Union,
 )
 
-from cassandra import CoordinationFailure
+from cassandra import ConsistencyLevel, CoordinationFailure
 from cassandra.cluster import EXEC_PROFILE_DEFAULT, Cluster, ExecutionProfile, ResultSet
+from cassandra.concurrent import execute_concurrent_with_args
 from cassandra.policies import DCAwareRoundRobinPolicy, TokenAwarePolicy
 from cassandra.query import BoundStatement, PreparedStatement, dict_factory
 from mypy_extensions import NamedArg
@@ -35,6 +38,7 @@ from tenacity import (
     wait_random_exponential,
 )
 
+from swh.core.utils import grouper
 from swh.model.identifiers import CoreSWHID
 from swh.model.model import (
     Content,
@@ -62,6 +66,7 @@ from .model import (
     OriginRow,
     OriginVisitRow,
     OriginVisitStatusRow,
+    RawExtrinsicMetadataByIdRow,
     RawExtrinsicMetadataRow,
     ReleaseRow,
     RevisionParentRow,
@@ -73,15 +78,39 @@ from .model import (
 )
 from .schema import CREATE_TABLES_QUERIES, HASH_ALGORITHMS
 
+PARTITION_KEY_RESTRICTION_MAX_SIZE = 100
+"""Maximum number of restrictions in a single query.
+Usually this is a very low number (eg. SELECT ... FROM ... WHERE x=?),
+but some queries can request arbitrarily many (eg. SELECT ... FROM ... WHERE x IN ?).
+
+This can cause performance issues, as the node getting the query need to
+coordinate with other nodes to get the complete results.
+See <https://github.com/scylladb/scylla/pull/4797> for details and rationale.
+"""
+
+BATCH_INSERT_MAX_SIZE = 1000
+
+
 logger = logging.getLogger(__name__)
 
 
-_execution_profiles = {
-    EXEC_PROFILE_DEFAULT: ExecutionProfile(
-        load_balancing_policy=TokenAwarePolicy(DCAwareRoundRobinPolicy()),
-        row_factory=dict_factory,
-    ),
-}
+def get_execution_profiles(
+    consistency_level: str = "ONE",
+) -> Dict[object, ExecutionProfile]:
+    if consistency_level not in ConsistencyLevel.name_to_value:
+        raise ValueError(
+            f"Configuration error: Unknown consistency level '{consistency_level}'"
+        )
+
+    return {
+        EXEC_PROFILE_DEFAULT: ExecutionProfile(
+            load_balancing_policy=TokenAwarePolicy(DCAwareRoundRobinPolicy()),
+            row_factory=dict_factory,
+            consistency_level=ConsistencyLevel.name_to_value[consistency_level],
+        )
+    }
+
+
 # Configuration for cassandra-driver's access to servers:
 # * hit the right server directly when sending a query (TokenAwarePolicy),
 # * if there's more than one, then pick one at random that's in the same
@@ -91,7 +120,7 @@ _execution_profiles = {
 def create_keyspace(
     hosts: List[str], keyspace: str, port: int = 9042, *, durable_writes=True
 ):
-    cluster = Cluster(hosts, port=port, execution_profiles=_execution_profiles)
+    cluster = Cluster(hosts, port=port, execution_profiles=get_execution_profiles())
     session = cluster.connect()
     extra_params = ""
     if not durable_writes:
@@ -142,6 +171,14 @@ TArg = TypeVar("TArg")
 TSelf = TypeVar("TSelf")
 
 
+def _insert_query(row_class):
+    columns = row_class.cols()
+    return (
+        f"INSERT INTO {row_class.TABLE} ({', '.join(columns)}) "
+        f"VALUES ({', '.join('?' for _ in columns)})"
+    )
+
+
 def _prepared_insert_statement(
     row_class: Type[BaseRow],
 ) -> Callable[
@@ -150,11 +187,7 @@ def _prepared_insert_statement(
 ]:
     """Shorthand for using `_prepared_statement` for `INSERT INTO`
     statements."""
-    columns = row_class.cols()
-    return _prepared_statement(
-        "INSERT INTO %s (%s) VALUES (%s)"
-        % (row_class.TABLE, ", ".join(columns), ", ".join("?" for _ in columns),)
-    )
+    return _prepared_statement(_insert_query(row_class))
 
 
 def _prepared_exists_statement(
@@ -222,9 +255,13 @@ class CqlRunner:
     """Class managing prepared statements and building queries to be sent
     to Cassandra."""
 
-    def __init__(self, hosts: List[str], keyspace: str, port: int):
+    def __init__(
+        self, hosts: List[str], keyspace: str, port: int, consistency_level: str
+    ):
         self._cluster = Cluster(
-            hosts, port=port, execution_profiles=_execution_profiles
+            hosts,
+            port=port,
+            execution_profiles=get_execution_profiles(consistency_level),
         )
         self._session = self._cluster.connect(keyspace)
         self._cluster.register_user_type(
@@ -252,22 +289,31 @@ class CqlRunner:
         stop=stop_after_attempt(MAX_RETRIES),
         retry=retry_if_exception_type(CoordinationFailure),
     )
-    def _execute_with_retries(self, statement, args) -> ResultSet:
+    def _execute_with_retries(self, statement, args: Optional[Sequence]) -> ResultSet:
         return self._session.execute(statement, args, timeout=1000.0)
 
-    @_prepared_statement(
-        "UPDATE object_count SET count = count + ? "
-        "WHERE partition_key = 0 AND object_type = ?"
+    @retry(
+        wait=wait_random_exponential(multiplier=1, max=10),
+        stop=stop_after_attempt(MAX_RETRIES),
+        retry=retry_if_exception_type(CoordinationFailure),
     )
-    def _increment_counter(
-        self, object_type: str, nb: int, *, statement: PreparedStatement
-    ) -> None:
-        self._execute_with_retries(statement, [nb, object_type])
+    def _execute_many_with_retries(
+        self, statement, args_list: List[Tuple]
+    ) -> Iterable[BaseRow]:
+        for res in execute_concurrent_with_args(self._session, statement, args_list):
+            yield from res.result_or_exc
 
     def _add_one(self, statement, obj: BaseRow) -> None:
-        self._increment_counter(obj.TABLE, 1)
         self._execute_with_retries(statement, dataclasses.astuple(obj))
 
+    def _add_many(self, statement, objs: Sequence[BaseRow]) -> None:
+        tables = {obj.TABLE for obj in objs}
+        assert len(tables) == 1, f"Cannot insert to multiple tables: {tables}"
+        rows = list(map(dataclasses.astuple, objs))
+        for _ in self._execute_many_with_retries(statement, rows):
+            # Need to consume the generator to actually run the INSERTs
+            pass
+
     _T = TypeVar("_T", bound=BaseRow)
 
     def _get_random_row(self, row_class: Type[_T], statement) -> Optional[_T]:  # noqa
@@ -286,8 +332,10 @@ class CqlRunner:
             return None
 
     def _missing(self, statement, ids):
-        rows = self._execute_with_retries(statement, [ids])
-        found_ids = {row["id"] for row in rows}
+        found_ids = set()
+        for id_group in grouper(ids, PARTITION_KEY_RESTRICTION_MAX_SIZE):
+            rows = self._execute_with_retries(statement, [list(id_group)])
+            found_ids.update(row["id"] for row in rows)
         return [id_ for id_ in ids if id_ not in found_ids]
 
     ##########################
@@ -298,7 +346,6 @@ class CqlRunner:
         """Returned currified by content_add_prepare, to be called when the
         content row should be added to the primary table."""
         self._execute_with_retries(statement, None)
-        self._increment_counter("content", 1)
 
     @_prepared_insert_statement(ContentRow)
     def content_add_prepare(
@@ -343,6 +390,37 @@ class CqlRunner:
         else:
             return None
 
+    def content_missing_from_hashes(
+        self, contents_hashes: List[Dict[str, bytes]]
+    ) -> Iterator[Dict[str, bytes]]:
+        for group in grouper(contents_hashes, PARTITION_KEY_RESTRICTION_MAX_SIZE):
+            group = list(group)
+
+            # Get all contents that share a sha256 with one of the contents in the group
+            present = set(
+                self._content_get_hashes_from_sha256(
+                    [content["sha256"] for content in group]
+                )
+            )
+
+            for content in group:
+                for algo in HASH_ALGORITHMS:
+                    assert content.get(algo) is not None, (
+                        "content_missing_from_hashes must not be called with "
+                        "partial hashes."
+                    )
+                if tuple(content[algo] for algo in HASH_ALGORITHMS) not in present:
+                    yield content
+
+    @_prepared_statement(
+        f"SELECT {', '.join(HASH_ALGORITHMS)} FROM content WHERE sha256 IN ?"
+    )
+    def _content_get_hashes_from_sha256(
+        self, ids: List[bytes], *, statement
+    ) -> Iterator[Tuple[bytes, bytes, bytes, bytes]]:
+        for row in self._execute_with_retries(statement, [ids]):
+            yield tuple(row[algo] for algo in HASH_ALGORITHMS)  # type: ignore
+
     @_prepared_select_statement(
         ContentRow, f"WHERE token({', '.join(ContentRow.PARTITION_KEY)}) = ?"
     )
@@ -400,8 +478,8 @@ class CqlRunner:
         """
         self._execute_with_retries(query, [content.get_hash(algo), token])
 
-    def content_get_tokens_from_single_hash(
-        self, algo: str, hash_: bytes
+    def content_get_tokens_from_single_algo(
+        self, algo: str, hashes: List[bytes]
     ) -> Iterable[int]:
         assert algo in HASH_ALGORITHMS
         query = f"""
@@ -410,7 +488,10 @@ class CqlRunner:
             WHERE {algo} = %s
         """
         return (
-            row["target_token"] for row in self._execute_with_retries(query, [hash_])
+            row["target_token"]  # type: ignore
+            for row in self._execute_many_with_retries(
+                query, [(hash_,) for hash_ in hashes]
+            )
         )
 
     ##########################
@@ -421,7 +502,6 @@ class CqlRunner:
         """Returned currified by skipped_content_add_prepare, to be called
         when the content row should be added to the primary table."""
         self._execute_with_retries(statement, None)
-        self._increment_counter("skipped_content", 1)
 
     @_prepared_insert_statement(SkippedContentRow)
     def skipped_content_add_prepare(
@@ -616,6 +696,49 @@ class CqlRunner:
     def directory_entry_add_one(self, entry: DirectoryEntryRow, *, statement) -> None:
         self._add_one(statement, entry)
 
+    @_prepared_insert_statement(DirectoryEntryRow)
+    def directory_entry_add_concurrent(
+        self, entries: List[DirectoryEntryRow], *, statement
+    ) -> None:
+        if len(entries) == 0:
+            # nothing to do
+            return
+        assert (
+            len({entry.directory_id for entry in entries}) == 1
+        ), "directory_entry_add_many must be called with entries for a single dir"
+        self._add_many(statement, entries)
+
+    @_prepared_statement(
+        "BEGIN UNLOGGED BATCH\n"
+        + (_insert_query(DirectoryEntryRow) + ";\n") * BATCH_INSERT_MAX_SIZE
+        + "APPLY BATCH"
+    )
+    def directory_entry_add_batch(
+        self, entries: List[DirectoryEntryRow], *, statement
+    ) -> None:
+        if len(entries) == 0:
+            # nothing to do
+            return
+        assert (
+            len({entry.directory_id for entry in entries}) == 1
+        ), "directory_entry_add_many must be called with entries for a single dir"
+
+        for entry_group in grouper(entries, BATCH_INSERT_MAX_SIZE):
+            entry_group = list(entry_group)
+            if len(entry_group) == BATCH_INSERT_MAX_SIZE:
+                entry_group = list(map(dataclasses.astuple, entry_group))
+                self._execute_with_retries(
+                    statement, list(itertools.chain.from_iterable(entry_group))
+                )
+            else:
+                # Last group, with a smaller size than the BATCH we prepared.
+                # Creating a prepared BATCH just for this then discarding it would
+                # create too much churn on the server side; and using unprepared
+                # statements is annoying (we can't use _insert_query() as they have
+                # a different format)
+                # Fall back to inserting concurrently.
+                self.directory_entry_add_concurrent(entry_group)
+
     @_prepared_select_statement(DirectoryEntryRow, "WHERE directory_id IN ?")
     def directory_entry_get(
         self, directory_ids, *, statement
@@ -625,6 +748,17 @@ class CqlRunner:
             self._execute_with_retries(statement, [directory_ids]),
         )
 
+    @_prepared_select_statement(
+        DirectoryEntryRow, "WHERE directory_id = ? AND name >= ? LIMIT ?"
+    )
+    def directory_entry_get_from_name(
+        self, directory_id: Sha1Git, from_: bytes, limit: int, *, statement
+    ) -> Iterable[DirectoryEntryRow]:
+        return map(
+            DirectoryEntryRow.from_dict,
+            self._execute_with_retries(statement, [directory_id, from_, limit]),
+        )
+
     ##########################
     # 'snapshot' table
     ##########################
@@ -791,6 +925,20 @@ class CqlRunner:
     def origin_iter_all(self, *, statement) -> Iterable[OriginRow]:
         return map(OriginRow.from_dict, self._execute_with_retries(statement, []))
 
+    @_prepared_statement(
+        f"""
+        UPDATE {OriginRow.TABLE}
+        SET next_visit_id=?
+        WHERE sha1 = ? IF next_visit_id<?
+        """
+    )
+    def origin_bump_next_visit_id(
+        self, origin_url: str, visit_id: int, *, statement
+    ) -> None:
+        origin_sha1 = hash_url(origin_url)
+        next_id = visit_id + 1
+        self._execute_with_retries(statement, [next_id, origin_sha1, next_id])
+
     @_prepared_statement(f"SELECT next_visit_id FROM {OriginRow.TABLE} WHERE sha1 = ?")
     def _origin_get_next_visit_id(self, origin_sha1: bytes, *, statement) -> int:
         rows = list(self._execute_with_retries(statement, [origin_sha1]))
@@ -969,7 +1117,9 @@ class CqlRunner:
         return next(self.origin_visit_status_get(origin, visit), None)
 
     @_prepared_select_statement(
-        OriginVisitStatusRow, "WHERE origin = ? AND visit = ? ORDER BY date DESC"
+        OriginVisitStatusRow,
+        # 'visit DESC,' is optional with Cassandra 4, but ScyllaDB needs it
+        "WHERE origin = ? AND visit = ? ORDER BY visit DESC, date DESC",
     )
     def origin_visit_status_get(
         self, origin: str, visit: int, *, statement,
@@ -1018,6 +1168,23 @@ class CqlRunner:
         else:
             return None
 
+    #########################
+    # 'raw_extrinsic_metadata_by_id' table
+    #########################
+
+    @_prepared_insert_statement(RawExtrinsicMetadataByIdRow)
+    def raw_extrinsic_metadata_by_id_add(self, row, *, statement):
+        self._add_one(statement, row)
+
+    @_prepared_select_statement(RawExtrinsicMetadataByIdRow, "WHERE id IN ?")
+    def raw_extrinsic_metadata_get_by_ids(
+        self, ids: List[Sha1Git], *, statement
+    ) -> Iterable[RawExtrinsicMetadataByIdRow]:
+        return map(
+            RawExtrinsicMetadataByIdRow.from_dict,
+            self._execute_with_retries(statement, [ids]),
+        )
+
     #########################
     # 'raw_extrinsic_metadata' table
     #########################
@@ -1048,8 +1215,12 @@ class CqlRunner:
 
     @_prepared_select_statement(
         RawExtrinsicMetadataRow,
-        "WHERE target=? AND authority_type=? AND authority_url=? "
-        "AND (discovery_date, id) > (?, ?)",
+        # This is equivalent to:
+        #   WHERE target=? AND authority_type = ? AND authority_url = ? "
+        #   AND (discovery_date, id) > (?, ?)"
+        # but it needs to be written this way to work with ScyllaDB.
+        "WHERE target=? AND (authority_type, authority_url) <= (?, ?) "
+        "AND (authority_type, authority_url, discovery_date, id) > (?, ?, ?, ?)",
     )
     def raw_extrinsic_metadata_get_after_date_and_id(
         self,
@@ -1065,7 +1236,15 @@ class CqlRunner:
             RawExtrinsicMetadataRow.from_dict,
             self._execute_with_retries(
                 statement,
-                [target, authority_type, authority_url, after_date, after_id,],
+                [
+                    target,
+                    authority_type,
+                    authority_url,
+                    authority_type,
+                    authority_url,
+                    after_date,
+                    after_id,
+                ],
             ),
         )
 
@@ -1083,6 +1262,18 @@ class CqlRunner:
             ),
         )
 
+    @_prepared_statement(
+        "SELECT authority_type, authority_url FROM raw_extrinsic_metadata "
+        "WHERE target = ?"
+    )
+    def raw_extrinsic_metadata_get_authorities(
+        self, target: str, *, statement
+    ) -> Iterable[Tuple[str, str]]:
+        return (
+            (entry["authority_type"], entry["authority_url"])
+            for entry in self._execute_with_retries(statement, [target])
+        )
+
     ##########################
     # 'extid' table
     ##########################
@@ -1090,7 +1281,6 @@ class CqlRunner:
         """Returned currified by extid_add_prepare, to be called when the
         extid row should be added to the primary table."""
         self._execute_with_retries(statement, None)
-        self._increment_counter("extid", 1)
 
     @_prepared_insert_statement(ExtIDRow)
     def extid_add_prepare(
@@ -1108,15 +1298,29 @@ class CqlRunner:
         return (token, finalizer)
 
     @_prepared_select_statement(
-        ExtIDRow, "WHERE extid_type=? AND extid=? AND target_type=? AND target=?",
+        ExtIDRow,
+        "WHERE extid_type=? AND extid=? AND extid_version=? "
+        "AND target_type=? AND target=?",
     )
     def extid_get_from_pk(
-        self, extid_type: str, extid: bytes, target: CoreSWHID, *, statement,
+        self,
+        extid_type: str,
+        extid: bytes,
+        extid_version: int,
+        target: CoreSWHID,
+        *,
+        statement,
     ) -> Optional[ExtIDRow]:
         rows = list(
             self._execute_with_retries(
                 statement,
-                [extid_type, extid, target.object_type.value, target.object_id],
+                [
+                    extid_type,
+                    extid,
+                    extid_version,
+                    target.object_type.value,
+                    target.object_id,
+                ],
             ),
         )
         assert len(rows) <= 1
@@ -1131,6 +1335,22 @@ class CqlRunner:
     def extid_get_from_token(self, token: int, *, statement) -> Iterable[ExtIDRow]:
         return map(ExtIDRow.from_dict, self._execute_with_retries(statement, [token]),)
 
+    # Rows are partitioned by token(extid_type, extid), then ordered (aka. "clustered")
+    # by (extid_type, extid, extid_version, ...). This means that, without knowing the
+    # exact extid_type and extid, we need to scan the whole partition; which should be
+    # reasonably small. We can change the schema later if this becomes an issue
+    @_prepared_select_statement(
+        ExtIDRow,
+        "WHERE token(extid_type, extid) = ? AND extid_version = ? ALLOW FILTERING",
+    )
+    def extid_get_from_token_and_extid_version(
+        self, token: int, extid_version: int, *, statement
+    ) -> Iterable[ExtIDRow]:
+        return map(
+            ExtIDRow.from_dict,
+            self._execute_with_retries(statement, [token, extid_version]),
+        )
+
     @_prepared_select_statement(
         ExtIDRow, "WHERE extid_type=? AND extid=?",
     )
@@ -1142,17 +1362,50 @@ class CqlRunner:
             self._execute_with_retries(statement, [extid_type, extid]),
         )
 
+    @_prepared_select_statement(
+        ExtIDRow, "WHERE extid_type=? AND extid=? AND extid_version = ?",
+    )
+    def extid_get_from_extid_and_version(
+        self, extid_type: str, extid: bytes, extid_version: int, *, statement
+    ) -> Iterable[ExtIDRow]:
+        return map(
+            ExtIDRow.from_dict,
+            self._execute_with_retries(statement, [extid_type, extid, extid_version]),
+        )
+
     def extid_get_from_target(
-        self, target_type: str, target: bytes
+        self,
+        target_type: str,
+        target: bytes,
+        extid_type: Optional[str] = None,
+        extid_version: Optional[int] = None,
     ) -> Iterable[ExtIDRow]:
         for token in self._extid_get_tokens_from_target(target_type, target):
             if token is not None:
-                for extid in self.extid_get_from_token(token):
+                if extid_type is not None and extid_version is not None:
+                    extids = self.extid_get_from_token_and_extid_version(
+                        token, extid_version
+                    )
+                else:
+                    extids = self.extid_get_from_token(token)
+
+                for extid in extids:
                     # re-check the extid against target (in case of murmur3 collision)
                     if (
                         extid is not None
                         and extid.target_type == target_type
                         and extid.target == target
+                        and (
+                            (extid_version is None and extid_type is None)
+                            or (
+                                (
+                                    extid_version is not None
+                                    and extid.extid_version == extid_version
+                                    and extid_type is not None
+                                    and extid.extid_type == extid_type
+                                )
+                            )
+                        )
                     ):
                         yield extid
 
@@ -1185,10 +1438,11 @@ class CqlRunner:
     # Miscellaneous
     ##########################
 
+    def stat_counters(self) -> Iterable[ObjectCountRow]:
+        raise NotImplementedError(
+            "stat_counters is not implemented by the Cassandra backend"
+        )
+
     @_prepared_statement("SELECT uuid() FROM revision LIMIT 1;")
     def check_read(self, *, statement):
         self._execute_with_retries(statement, [])
-
-    @_prepared_select_statement(ObjectCountRow, "WHERE partition_key=0")
-    def stat_counters(self, *, statement) -> Iterable[ObjectCountRow]:
-        return map(ObjectCountRow.from_dict, self._execute_with_retries(statement, []))
