@@ -68,7 +68,12 @@ class BaseRow:
 @dataclasses.dataclass
 class ContentRow(BaseRow):
     TABLE = "content"
-    PARTITION_KEY = ("sha1", "sha1_git", "sha256", "blake2s256")
+    PARTITION_KEY: ClassVar[Tuple[str, ...]] = ("sha256",)
+    CLUSTERING_KEY = (
+        "sha1",
+        "sha1_git",
+        "blake2s256",
+    )
 
     sha1: bytes
     sha1_git: bytes
@@ -237,7 +242,6 @@ class MetadataAuthorityRow(BaseRow):
 
     url: str
     type: str
-    metadata: str
 
 
 @dataclasses.dataclass
@@ -248,7 +252,6 @@ class MetadataFetcherRow(BaseRow):
 
     name: str
     version: str
-    metadata: str
 
 
 @dataclasses.dataclass
@@ -285,6 +288,18 @@ class RawExtrinsicMetadataRow(BaseRow):
     directory: Optional[str]
 
 
+@dataclasses.dataclass
+class RawExtrinsicMetadataByIdRow(BaseRow):
+    TABLE = "raw_extrinsic_metadata_by_id"
+    PARTITION_KEY = ("id",)
+    CLUSTERING_KEY = ()
+
+    id: bytes
+    target: str
+    authority_type: str
+    authority_url: str
+
+
 @dataclasses.dataclass
 class ObjectCountRow(BaseRow):
     TABLE = "object_count"
@@ -299,10 +314,11 @@ class ObjectCountRow(BaseRow):
 @dataclasses.dataclass
 class ExtIDRow(BaseRow):
     TABLE = "extid"
-    PARTITION_KEY = ("target", "target_type", "extid", "extid_type")
+    PARTITION_KEY = ("target", "target_type", "extid_version", "extid", "extid_type")
 
     extid_type: str
     extid: bytes
+    extid_version: int
     target_type: str
     target: bytes
 
