@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -6,6 +6,7 @@
 from collections import defaultdict
 import datetime
 import functools
+import itertools
 import random
 from typing import (
     Any,
@@ -37,6 +38,7 @@ from swh.storage.cassandra.model import (
     OriginRow,
     OriginVisitRow,
     OriginVisitStatusRow,
+    RawExtrinsicMetadataByIdRow,
     RawExtrinsicMetadataRow,
     ReleaseRow,
     RevisionParentRow,
@@ -167,6 +169,7 @@ class InMemoryCqlRunner:
         self._metadata_authorities = Table(MetadataAuthorityRow)
         self._metadata_fetchers = Table(MetadataFetcherRow)
         self._raw_extrinsic_metadata = Table(RawExtrinsicMetadataRow)
+        self._raw_extrinsic_metadata_by_id = Table(RawExtrinsicMetadataByIdRow)
         self._extid = Table(ExtIDRow)
         self._stat_counters = defaultdict(int)
 
@@ -213,6 +216,13 @@ class InMemoryCqlRunner:
         matches.sort()
         return matches[0:limit]
 
+    def content_missing_from_hashes(
+        self, contents_hashes: List[Dict[str, bytes]]
+    ) -> Iterator[Dict[str, bytes]]:
+        for content_hashes in contents_hashes:
+            if not self.content_get_from_pk(content_hashes):
+                yield content_hashes
+
     ##########################
     # 'content_by_*' tables
     ##########################
@@ -227,10 +237,11 @@ class InMemoryCqlRunner:
     def content_index_add_one(self, algo: str, content: Content, token: int) -> None:
         self._content_indexes[algo][content.get_hash(algo)].add(token)
 
-    def content_get_tokens_from_single_hash(
-        self, algo: str, hash_: bytes
+    def content_get_tokens_from_single_algo(
+        self, algo: str, hashes: List[bytes]
     ) -> Iterable[int]:
-        return self._content_indexes[algo][hash_]
+        for hash_ in hashes:
+            yield from self._content_indexes[algo][hash_]
 
     ##########################
     # 'skipped_content' table
@@ -301,6 +312,16 @@ class InMemoryCqlRunner:
         for id_ in directory_ids:
             yield from self._directory_entries.get_from_partition_key((id_,))
 
+    def directory_entry_get_from_name(
+        self, directory_id: Sha1Git, from_: bytes, limit: int
+    ) -> Iterable[DirectoryEntryRow]:
+        # Get all entries
+        entries = self._directory_entries.get_from_partition_key((directory_id,))
+        # Filter out the ones before from_
+        entries = itertools.dropwhile(lambda entry: entry.name < from_, entries)
+        # Apply limit
+        return itertools.islice(entries, limit)
+
     ##########################
     # 'revision' table
     ##########################
@@ -460,6 +481,10 @@ class InMemoryCqlRunner:
             for (clustering_key, row) in partition.items()
         )
 
+    def origin_bump_next_visit_id(self, origin_url: str, visit_id: int) -> None:
+        origin = list(self.origin_get_by_url(origin_url))[0]
+        origin.next_visit_id = max(origin.next_visit_id, visit_id + 1)
+
     def origin_generate_unique_visit_id(self, origin_url: str) -> int:
         origin = list(self.origin_get_by_url(origin_url))[0]
         visit_id = origin.next_visit_id
@@ -580,6 +605,25 @@ class InMemoryCqlRunner:
     def metadata_fetcher_get(self, name, version) -> Optional[MetadataAuthorityRow]:
         return self._metadata_fetchers.get_from_primary_key((name, version))
 
+    #########################
+    # 'raw_extrinsic_metadata_by_id' table
+    #########################
+
+    def raw_extrinsic_metadata_by_id_add(
+        self, row: RawExtrinsicMetadataByIdRow
+    ) -> None:
+        self._raw_extrinsic_metadata_by_id.insert(row)
+
+    def raw_extrinsic_metadata_get_by_ids(
+        self, ids
+    ) -> List[RawExtrinsicMetadataByIdRow]:
+        results = []
+        for id_ in ids:
+            result = self._raw_extrinsic_metadata_by_id.get_from_primary_key((id_,))
+            if result:
+                results.append(result)
+        return results
+
     #########################
     # 'raw_extrinsic_metadata' table
     #########################
@@ -628,6 +672,12 @@ class InMemoryCqlRunner:
             if m.authority_type == authority_type and m.authority_url == authority_url
         )
 
+    def raw_extrinsic_metadata_get_authorities(
+        self, target: str
+    ) -> Iterable[Tuple[str, str]]:
+        metadata = self._raw_extrinsic_metadata.get_from_partition_key((target,))
+        return ((m.authority_type, m.authority_url) for m in metadata)
+
     #########################
     # 'extid' table
     #########################
@@ -643,27 +693,53 @@ class InMemoryCqlRunner:
         pass
 
     def extid_get_from_pk(
-        self, extid_type: str, extid: bytes, target: ExtendedSWHID,
+        self, extid_type: str, extid: bytes, extid_version: int, target: ExtendedSWHID,
     ) -> Optional[ExtIDRow]:
         primary_key = self._extid.primary_key_from_dict(
             dict(
                 extid_type=extid_type,
                 extid=extid,
+                extid_version=extid_version,
                 target_type=target.object_type.value,
                 target=target.object_id,
             )
         )
         return self._extid.get_from_primary_key(primary_key)
 
-    def extid_get_from_extid(self, extid_type: str, extid: bytes) -> Iterable[ExtIDRow]:
+    def extid_get_from_extid(
+        self, extid_type: str, extid: bytes,
+    ) -> Iterable[ExtIDRow]:
         return (
             row
             for pk, row in self._extid.iter_all()
             if row.extid_type == extid_type and row.extid == extid
         )
 
-    def extid_get_from_target(
-        self, target_type: str, target: bytes
+    def extid_get_from_extid_and_version(
+        self, extid_type: str, extid: bytes, extid_version: int,
+    ) -> Iterable[ExtIDRow]:
+        return (
+            row
+            for pk, row in self._extid.iter_all()
+            if row.extid_type == extid_type
+            and row.extid == extid
+            and (extid_version is None or row.extid_version == extid_version)
+        )
+
+    def _extid_get_from_target_with_type_and_version(
+        self, target_type: str, target: bytes, extid_type: str, extid_version: int,
+    ) -> Iterable[ExtIDRow]:
+        return (
+            row
+            for pk, row in self._extid.iter_all()
+            if row.target_type == target_type
+            and row.target == target
+            and row.extid_version == extid_version
+            and row.extid_type == extid_type
+        )
+
+    def _extid_get_from_target(
+        self, target_type: str, target: bytes,
     ) -> Iterable[ExtIDRow]:
         return (
             row
@@ -671,6 +747,26 @@ class InMemoryCqlRunner:
             if row.target_type == target_type and row.target == target
         )
 
+    def extid_get_from_target(
+        self,
+        target_type: str,
+        target: bytes,
+        extid_type: Optional[str] = None,
+        extid_version: Optional[int] = None,
+    ) -> Iterable[ExtIDRow]:
+        if (extid_version is not None and extid_type is None) or (
+            extid_version is None and extid_type is not None
+        ):
+            raise ValueError("You must provide both extid_type and extid_version")
+
+        if extid_type is not None and extid_version is not None:
+            extids = self._extid_get_from_target_with_type_and_version(
+                target_type, target, extid_type, extid_version
+            )
+        else:
+            extids = self._extid_get_from_target(target_type, target)
+        return extids
+
 
 class InMemoryStorage(CassandraStorage):
     _cql_runner: InMemoryCqlRunner  # type: ignore
@@ -678,6 +774,8 @@ class InMemoryStorage(CassandraStorage):
     def __init__(self, journal_writer=None):
         self.reset()
         self.journal_writer = JournalWriter(journal_writer)
+        self._allow_overwrite = False
+        self._directory_entries_insert_algo = "one-by-one"
 
     def reset(self):
         self._cql_runner = InMemoryCqlRunner()
