@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -13,8 +13,9 @@ from swh.core.db import BaseDb
 from swh.core.db.db_utils import execute_values_generator
 from swh.core.db.db_utils import jsonize as _jsonize
 from swh.core.db.db_utils import stored_procedure
+from swh.model.hashutil import DEFAULT_ALGORITHMS
 from swh.model.identifiers import ObjectType
-from swh.model.model import SHA1_SIZE, OriginVisit, OriginVisitStatus
+from swh.model.model import SHA1_SIZE, OriginVisit, OriginVisitStatus, Sha1Git
 from swh.storage.interface import ListOrder
 
 logger = logging.getLogger(__name__)
@@ -29,7 +30,7 @@ class Db(BaseDb):
 
     """
 
-    current_version = 173
+    current_version = 177
 
     def mktemp_dir_entry(self, entry_type, cur=None):
         self._cursor(cur).execute(
@@ -113,16 +114,18 @@ class Db(BaseDb):
         "origin",
     ]
 
-    def content_get_metadata_from_sha1s(self, sha1s, cur=None):
+    def content_get_metadata_from_hashes(
+        self, hashes: List[bytes], algo: str, cur=None
+    ):
         cur = self._cursor(cur)
+        assert algo in DEFAULT_ALGORITHMS
+        query = f"""
+            select {", ".join(self.content_get_metadata_keys)}
+            from (values %s) as t (hash)
+            inner join content on (content.{algo}=hash)
+        """
         yield from execute_values_generator(
-            cur,
-            """
-            select t.sha1, %s from (values %%s) as t (sha1)
-            inner join content using (sha1)
-            """
-            % ", ".join(self.content_get_metadata_keys[1:]),
-            ((sha1,) for sha1 in sha1s),
+            cur, query, ((hash_,) for hash_ in hashes),
         )
 
     def content_get_range(self, start, end, limit=None, cur=None):
@@ -403,6 +406,15 @@ class Db(BaseDb):
             return None
         return data
 
+    directory_get_entries_cols = ["type", "target", "name", "perms"]
+
+    def directory_get_entries(self, directory: Sha1Git, cur=None) -> List[Tuple]:
+        cur = self._cursor(cur)
+        cur.execute(
+            "SELECT * FROM swh_directory_get_entries(%s::sha1_git)", (directory,)
+        )
+        return list(cur)
+
     def directory_get_random(self, cur=None):
         return self._get_random_row_from_table("directory", ["id"], "id", cur)
 
@@ -526,8 +538,8 @@ class Db(BaseDb):
         "ov.date",
         "ov.type AS type",
         "ovs.status",
-        "ovs.metadata",
         "ovs.snapshot",
+        "ovs.metadata",
     ]
 
     origin_visit_status_select_cols = [
@@ -831,21 +843,28 @@ class Db(BaseDb):
             ((sortkey, id) for sortkey, id in enumerate(revisions)),
         )
 
-    extid_cols = ["extid", "extid_type", "target", "target_type"]
+    extid_cols = ["extid", "extid_version", "extid_type", "target", "target_type"]
 
-    def extid_get_from_extid_list(self, extid_type, ids, cur=None):
+    def extid_get_from_extid_list(
+        self, extid_type: str, ids: List[bytes], version: Optional[int] = None, cur=None
+    ):
         cur = self._cursor(cur)
         query_keys = ", ".join(
             self.mangle_query_key(k, "extid") for k in self.extid_cols
         )
-        sql = """
-            SELECT %s
-            FROM (VALUES %%s) as t(sortkey, extid, extid_type)
+        filter_query = ""
+        if version is not None:
+            filter_query = cur.mogrify(
+                f"WHERE extid_version={version}", (version,)
+            ).decode()
+
+        sql = f"""
+            SELECT {query_keys}
+            FROM (VALUES %s) as t(sortkey, extid, extid_type)
             LEFT JOIN extid USING (extid, extid_type)
+            {filter_query}
             ORDER BY sortkey
-            """ % (
-            query_keys,
-        )
+            """
 
         yield from execute_values_generator(
             cur,
@@ -853,7 +872,14 @@ class Db(BaseDb):
             (((sortkey, extid, extid_type) for sortkey, extid in enumerate(ids))),
         )
 
-    def extid_get_from_swhid_list(self, target_type, ids, cur=None):
+    def extid_get_from_swhid_list(
+        self,
+        target_type: str,
+        ids: List[bytes],
+        extid_version: Optional[int] = None,
+        extid_type: Optional[str] = None,
+        cur=None,
+    ):
         cur = self._cursor(cur)
         target_type = ObjectType(
             target_type
@@ -861,14 +887,20 @@ class Db(BaseDb):
         query_keys = ", ".join(
             self.mangle_query_key(k, "extid") for k in self.extid_cols
         )
-        sql = """
-            SELECT %s
-            FROM (VALUES %%s) as t(sortkey, target, target_type)
+        filter_query = ""
+        if extid_version is not None and extid_type is not None:
+            filter_query = cur.mogrify(
+                "WHERE extid_version=%s AND extid_type=%s", (extid_version, extid_type,)
+            ).decode()
+
+        sql = f"""
+            SELECT {query_keys}
+            FROM (VALUES %s) as t(sortkey, target, target_type)
             LEFT JOIN extid USING (target, target_type)
+            {filter_query}
             ORDER BY sortkey
-            """ % (
-            query_keys,
-        )
+            """
+
         yield from execute_values_generator(
             cur,
             sql,
@@ -971,10 +1003,11 @@ class Db(BaseDb):
     def origin_add(self, url, cur=None):
         """Insert a new origin and return the new identifier."""
         insert = """INSERT INTO origin (url) values (%s)
-                    RETURNING url"""
+                    ON CONFLICT DO NOTHING
+                    """
 
         cur.execute(insert, (url,))
-        return cur.fetchone()[0]
+        return cur.rowcount
 
     origin_cols = ["url"]
 
@@ -1253,7 +1286,6 @@ class Db(BaseDb):
         INNER JOIN metadata_authority
             ON (metadata_authority.id=authority_id)
         INNER JOIN metadata_fetcher ON (metadata_fetcher.id=fetcher_id)
-        WHERE raw_extrinsic_metadata.target=%s AND authority_id=%s
     """
 
     def raw_extrinsic_metadata_add(
@@ -1300,7 +1332,6 @@ class Db(BaseDb):
 
     def raw_extrinsic_metadata_get(
         self,
-        type: str,
         target: str,
         authority_id: int,
         after_time: Optional[datetime.datetime],
@@ -1309,6 +1340,7 @@ class Db(BaseDb):
         cur,
     ):
         query_parts = [self._raw_extrinsic_metadata_select_query]
+        query_parts.append("WHERE raw_extrinsic_metadata.target=%s AND authority_id=%s")
         args = [target, authority_id]
 
         if after_fetcher is not None:
@@ -1328,16 +1360,38 @@ class Db(BaseDb):
         cur.execute(" ".join(query_parts), args)
         yield from cur
 
-    metadata_fetcher_cols = ["name", "version", "metadata"]
+    def raw_extrinsic_metadata_get_by_ids(self, ids: List[Sha1Git], cur=None):
+        cur = self._cursor(cur)
+        yield from execute_values_generator(
+            cur,
+            self._raw_extrinsic_metadata_select_query
+            + "INNER JOIN (VALUES %s) AS t(id) ON t.id = raw_extrinsic_metadata.id",
+            [(id_,) for id_ in ids],
+        )
 
-    def metadata_fetcher_add(
-        self, name: str, version: str, metadata: bytes, cur=None
-    ) -> None:
+    def raw_extrinsic_metadata_get_authorities(self, id: str, cur=None):
         cur = self._cursor(cur)
         cur.execute(
-            "INSERT INTO metadata_fetcher (name, version, metadata) "
-            "VALUES (%s, %s, %s) ON CONFLICT DO NOTHING",
-            (name, version, jsonize(metadata)),
+            """
+            SELECT
+                DISTINCT metadata_authority.type, metadata_authority.url
+            FROM raw_extrinsic_metadata
+            INNER JOIN metadata_authority
+                ON (metadata_authority.id=authority_id)
+            WHERE raw_extrinsic_metadata.target = %s
+            """,
+            (id,),
+        )
+        yield from cur
+
+    metadata_fetcher_cols = ["name", "version"]
+
+    def metadata_fetcher_add(self, name: str, version: str, cur=None) -> None:
+        cur = self._cursor(cur)
+        cur.execute(
+            "INSERT INTO metadata_fetcher (name, version) "
+            "VALUES (%s, %s) ON CONFLICT DO NOTHING",
+            (name, version),
         )
 
     def metadata_fetcher_get(self, name: str, version: str, cur=None):
@@ -1364,16 +1418,14 @@ class Db(BaseDb):
         else:
             return None
 
-    metadata_authority_cols = ["type", "url", "metadata"]
+    metadata_authority_cols = ["type", "url"]
 
-    def metadata_authority_add(
-        self, type: str, url: str, metadata: bytes, cur=None
-    ) -> None:
+    def metadata_authority_add(self, type: str, url: str, cur=None) -> None:
         cur = self._cursor(cur)
         cur.execute(
-            "INSERT INTO metadata_authority (type, url, metadata) "
-            "VALUES (%s, %s, %s) ON CONFLICT DO NOTHING",
-            (type, url, jsonize(metadata)),
+            "INSERT INTO metadata_authority (type, url) "
+            "VALUES (%s, %s) ON CONFLICT DO NOTHING",
+            (type, url),
         )
 
     def metadata_authority_get(self, type: str, url: str, cur=None):
