@@ -9,6 +9,7 @@ import contextlib
 from contextlib import contextmanager
 import datetime
 import itertools
+import operator
 from typing import Any, Counter, Dict, Iterable, List, Optional, Sequence, Tuple
 
 import attr
@@ -24,6 +25,7 @@ from swh.model.model import (
     SHA1_SIZE,
     Content,
     Directory,
+    DirectoryEntry,
     ExtID,
     MetadataAuthority,
     MetadataAuthorityType,
@@ -144,7 +146,7 @@ class Storage:
 
     @timed
     @db_transaction()
-    def check_config(self, *, check_write: bool, db=None, cur=None) -> bool:
+    def check_config(self, *, check_write: bool, db: Db, cur=None) -> bool:
 
         if not self.objstorage.check_config(check_write=check_write):
             return False
@@ -210,7 +212,7 @@ class Storage:
 
     @timed
     @process_metrics
-    def content_add(self, content: List[Content]) -> Dict:
+    def content_add(self, content: List[Content]) -> Dict[str, int]:
         ctime = now()
 
         contents = [attr.evolve(c, ctime=ctime) for c in content]
@@ -245,7 +247,7 @@ class Storage:
     @timed
     @db_transaction()
     def content_update(
-        self, contents: List[Dict[str, Any]], keys: List[str] = [], db=None, cur=None
+        self, contents: List[Dict[str, Any]], keys: List[str] = [], *, db: Db, cur=None
     ) -> None:
         # TODO: Add a check on input keys. How to properly implement
         # this? We don't know yet the new columns.
@@ -260,7 +262,9 @@ class Storage:
     @timed
     @process_metrics
     @db_transaction()
-    def content_add_metadata(self, content: List[Content], db=None, cur=None) -> Dict:
+    def content_add_metadata(
+        self, content: List[Content], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         missing = self.content_missing(
             (c.to_dict() for c in content), key_hash="sha1_git", db=db, cur=cur,
         )
@@ -286,7 +290,8 @@ class Storage:
         nb_partitions: int,
         page_token: Optional[str] = None,
         limit: int = 1000,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> PagedResult[Content]:
         if limit is None:
@@ -316,20 +321,33 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def content_get(
-        self, contents: List[Sha1], db=None, cur=None
+        self, contents: List[bytes], algo: str = "sha1", *, db: Db, cur=None
     ) -> List[Optional[Content]]:
-        contents_by_sha1: Dict[Sha1, Optional[Content]] = {}
-        for row in db.content_get_metadata_from_sha1s(contents, cur):
+        contents_by_hash: Dict[bytes, Optional[Content]] = {}
+        if algo not in DEFAULT_ALGORITHMS:
+            raise StorageArgumentException(
+                "algo should be one of {','.join(DEFAULT_ALGORITHMS)}"
+            )
+
+        rows = db.content_get_metadata_from_hashes(contents, algo, cur)
+        key = operator.attrgetter(algo)
+
+        for row in rows:
             row_d = dict(zip(db.content_get_metadata_keys, row))
             content = Content(**row_d)
-            contents_by_sha1[content.sha1] = content
+            contents_by_hash[key(content)] = content
 
-        return [contents_by_sha1.get(sha1) for sha1 in contents]
+        return [contents_by_hash.get(sha1) for sha1 in contents]
 
     @timed
     @db_transaction_generator()
     def content_missing(
-        self, contents: List[Dict[str, Any]], key_hash: str = "sha1", db=None, cur=None
+        self,
+        contents: List[Dict[str, Any]],
+        key_hash: str = "sha1",
+        *,
+        db: Db,
+        cur=None,
     ) -> Iterable[bytes]:
         if key_hash not in DEFAULT_ALGORITHMS:
             raise StorageArgumentException(
@@ -345,7 +363,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def content_missing_per_sha1(
-        self, contents: List[bytes], db=None, cur=None
+        self, contents: List[bytes], *, db: Db, cur=None
     ) -> Iterable[bytes]:
         for obj in db.content_missing_per_sha1(contents, cur):
             yield obj[0]
@@ -353,14 +371,16 @@ class Storage:
     @timed
     @db_transaction_generator()
     def content_missing_per_sha1_git(
-        self, contents: List[bytes], db=None, cur=None
+        self, contents: List[bytes], *, db: Db, cur=None
     ) -> Iterable[Sha1Git]:
         for obj in db.content_missing_per_sha1_git(contents, cur):
             yield obj[0]
 
     @timed
     @db_transaction()
-    def content_find(self, content: Dict[str, Any], db=None, cur=None) -> List[Content]:
+    def content_find(
+        self, content: Dict[str, Any], *, db: Db, cur=None
+    ) -> List[Content]:
         if not set(content).intersection(DEFAULT_ALGORITHMS):
             raise StorageArgumentException(
                 "content keys must contain at least one "
@@ -382,7 +402,7 @@ class Storage:
 
     @timed
     @db_transaction()
-    def content_get_random(self, db=None, cur=None) -> Sha1Git:
+    def content_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.content_get_random(cur)
 
     @staticmethod
@@ -418,8 +438,8 @@ class Storage:
     @process_metrics
     @db_transaction()
     def skipped_content_add(
-        self, content: List[SkippedContent], db=None, cur=None
-    ) -> Dict:
+        self, content: List[SkippedContent], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         ctime = now()
         content = [attr.evolve(c, ctime=ctime) for c in content]
 
@@ -448,7 +468,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def skipped_content_missing(
-        self, contents: List[Dict[str, Any]], db=None, cur=None
+        self, contents: List[Dict[str, Any]], *, db: Db, cur=None
     ) -> Iterable[Dict[str, Any]]:
         contents = list(contents)
         for content in db.skipped_content_missing(contents, cur):
@@ -457,7 +477,9 @@ class Storage:
     @timed
     @process_metrics
     @db_transaction()
-    def directory_add(self, directories: List[Directory], db=None, cur=None) -> Dict:
+    def directory_add(
+        self, directories: List[Directory], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         summary = {"directory:add": 0}
 
         dirs = set()
@@ -514,7 +536,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def directory_missing(
-        self, directories: List[Sha1Git], db=None, cur=None
+        self, directories: List[Sha1Git], *, db: Db, cur=None
     ) -> Iterable[Sha1Git]:
         for obj in db.directory_missing_from_list(directories, cur):
             yield obj[0]
@@ -522,7 +544,7 @@ class Storage:
     @timed
     @db_transaction_generator(statement_timeout=20000)
     def directory_ls(
-        self, directory: Sha1Git, recursive: bool = False, db=None, cur=None
+        self, directory: Sha1Git, recursive: bool = False, *, db: Db, cur=None
     ) -> Iterable[Dict[str, Any]]:
         if recursive:
             res_gen = db.directory_walk(directory, cur=cur)
@@ -535,20 +557,48 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=2000)
     def directory_entry_get_by_path(
-        self, directory: Sha1Git, paths: List[bytes], db=None, cur=None
+        self, directory: Sha1Git, paths: List[bytes], *, db: Db, cur=None
     ) -> Optional[Dict[str, Any]]:
         res = db.directory_entry_get_by_path(directory, paths, cur)
         return dict(zip(db.directory_ls_cols, res)) if res else None
 
     @timed
     @db_transaction()
-    def directory_get_random(self, db=None, cur=None) -> Sha1Git:
+    def directory_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.directory_get_random(cur)
 
+    @db_transaction()
+    def directory_get_entries(
+        self,
+        directory_id: Sha1Git,
+        page_token: Optional[bytes] = None,
+        limit: int = 1000,
+        *,
+        db: Db,
+        cur=None,
+    ) -> Optional[PagedResult[DirectoryEntry]]:
+        if list(self.directory_missing([directory_id], db=db, cur=cur)):
+            return None
+
+        if page_token is not None:
+            raise StorageArgumentException("Unsupported page token")
+
+        # TODO: actually paginate
+        rows = db.directory_get_entries(directory_id, cur=cur)
+        return PagedResult(
+            results=[
+                DirectoryEntry(**dict(zip(db.directory_get_entries_cols, row)))
+                for row in rows
+            ],
+            next_page_token=None,
+        )
+
     @timed
     @process_metrics
     @db_transaction()
-    def revision_add(self, revisions: List[Revision], db=None, cur=None) -> Dict:
+    def revision_add(
+        self, revisions: List[Revision], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         summary = {"revision:add": 0}
 
         revisions_missing = set(
@@ -570,7 +620,7 @@ class Storage:
 
         db_revisions_filtered = list(map(converters.revision_to_db, revisions_filtered))
 
-        parents_filtered: List[bytes] = []
+        parents_filtered: List[Dict[str, Any]] = []
 
         with convert_validation_exceptions():
             db.copy_to(
@@ -595,7 +645,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def revision_missing(
-        self, revisions: List[Sha1Git], db=None, cur=None
+        self, revisions: List[Sha1Git], *, db: Db, cur=None
     ) -> Iterable[Sha1Git]:
         if not revisions:
             return None
@@ -606,7 +656,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=1000)
     def revision_get(
-        self, revision_ids: List[Sha1Git], db=None, cur=None
+        self, revision_ids: List[Sha1Git], *, db: Db, cur=None
     ) -> List[Optional[Revision]]:
         revisions = []
         for line in db.revision_get_from_list(revision_ids, cur):
@@ -618,7 +668,7 @@ class Storage:
     @timed
     @db_transaction_generator(statement_timeout=2000)
     def revision_log(
-        self, revisions: List[Sha1Git], limit: Optional[int] = None, db=None, cur=None
+        self, revisions: List[Sha1Git], limit: Optional[int] = None, *, db: Db, cur=None
     ) -> Iterable[Optional[Dict[str, Any]]]:
         for line in db.revision_log(revisions, limit, cur):
             data = converters.db_to_revision(dict(zip(db.revision_get_cols, line)))
@@ -630,22 +680,28 @@ class Storage:
     @timed
     @db_transaction_generator(statement_timeout=2000)
     def revision_shortlog(
-        self, revisions: List[Sha1Git], limit: Optional[int] = None, db=None, cur=None
+        self, revisions: List[Sha1Git], limit: Optional[int] = None, *, db: Db, cur=None
     ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
         yield from db.revision_shortlog(revisions, limit, cur)
 
     @timed
     @db_transaction()
-    def revision_get_random(self, db=None, cur=None) -> Sha1Git:
+    def revision_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.revision_get_random(cur)
 
     @timed
     @db_transaction()
     def extid_get_from_extid(
-        self, id_type: str, ids: List[bytes], db=None, cur=None
+        self,
+        id_type: str,
+        ids: List[bytes],
+        version: Optional[int] = None,
+        *,
+        db: Db,
+        cur=None,
     ) -> List[ExtID]:
         extids = []
-        for row in db.extid_get_from_extid_list(id_type, ids, cur):
+        for row in db.extid_get_from_extid_list(id_type, ids, version=version, cur=cur):
             if row[0] is not None:
                 extids.append(converters.db_to_extid(dict(zip(db.extid_cols, row))))
         return extids
@@ -653,21 +709,40 @@ class Storage:
     @timed
     @db_transaction()
     def extid_get_from_target(
-        self, target_type: ObjectType, ids: List[Sha1Git], db=None, cur=None
+        self,
+        target_type: ObjectType,
+        ids: List[Sha1Git],
+        extid_type: Optional[str] = None,
+        extid_version: Optional[int] = None,
+        *,
+        db: Db,
+        cur=None,
     ) -> List[ExtID]:
         extids = []
-        for row in db.extid_get_from_swhid_list(target_type.value, ids, cur):
+        if (extid_version is not None and extid_type is None) or (
+            extid_version is None and extid_type is not None
+        ):
+            raise ValueError("You must provide both extid_type and extid_version")
+
+        for row in db.extid_get_from_swhid_list(
+            target_type.value,
+            ids,
+            extid_version=extid_version,
+            extid_type=extid_type,
+            cur=cur,
+        ):
             if row[0] is not None:
                 extids.append(converters.db_to_extid(dict(zip(db.extid_cols, row))))
         return extids
 
     @timed
     @db_transaction()
-    def extid_add(self, ids: List[ExtID], db=None, cur=None) -> Dict[str, int]:
+    def extid_add(self, ids: List[ExtID], *, db: Db, cur=None) -> Dict[str, int]:
         extid = [
             {
                 "extid": extid.extid,
                 "extid_type": extid.extid_type,
+                "extid_version": getattr(extid, "extid_version", 0),
                 "target": extid.target.object_id,
                 "target_type": extid.target.object_type.name.lower(),  # arghh
             }
@@ -687,7 +762,9 @@ class Storage:
     @timed
     @process_metrics
     @db_transaction()
-    def release_add(self, releases: List[Release], db=None, cur=None) -> Dict:
+    def release_add(
+        self, releases: List[Release], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         summary = {"release:add": 0}
 
         release_ids = set(release.id for release in releases)
@@ -716,7 +793,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def release_missing(
-        self, releases: List[Sha1Git], db=None, cur=None
+        self, releases: List[Sha1Git], *, db: Db, cur=None
     ) -> Iterable[Sha1Git]:
         if not releases:
             return
@@ -727,7 +804,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def release_get(
-        self, releases: List[Sha1Git], db=None, cur=None
+        self, releases: List[Sha1Git], *, db: Db, cur=None
     ) -> List[Optional[Release]]:
         rels = []
         for release in db.release_get_from_list(releases, cur):
@@ -737,13 +814,15 @@ class Storage:
 
     @timed
     @db_transaction()
-    def release_get_random(self, db=None, cur=None) -> Sha1Git:
+    def release_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.release_get_random(cur)
 
     @timed
     @process_metrics
     @db_transaction()
-    def snapshot_add(self, snapshots: List[Snapshot], db=None, cur=None) -> Dict:
+    def snapshot_add(
+        self, snapshots: List[Snapshot], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         created_temp_table = False
 
         count = 0
@@ -780,7 +859,7 @@ class Storage:
     @timed
     @db_transaction_generator()
     def snapshot_missing(
-        self, snapshots: List[Sha1Git], db=None, cur=None
+        self, snapshots: List[Sha1Git], *, db: Db, cur=None
     ) -> Iterable[Sha1Git]:
         for obj in db.snapshot_missing_from_list(snapshots, cur):
             yield obj[0]
@@ -788,7 +867,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=2000)
     def snapshot_get(
-        self, snapshot_id: Sha1Git, db=None, cur=None
+        self, snapshot_id: Sha1Git, *, db: Db, cur=None
     ) -> Optional[Dict[str, Any]]:
         d = self.snapshot_get_branches(snapshot_id)
         if d is None:
@@ -808,7 +887,8 @@ class Storage:
         self,
         snapshot_id: Sha1Git,
         branch_name_exclude_prefix: Optional[bytes] = None,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> Optional[Dict[Optional[str], int]]:
         return dict(
@@ -830,7 +910,8 @@ class Storage:
         target_types: Optional[List[str]] = None,
         branch_name_include_substring: Optional[bytes] = None,
         branch_name_exclude_prefix: Optional[bytes] = None,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> Optional[PartialBranches]:
         if snapshot_id == EMPTY_SNAPSHOT_ID:
@@ -881,13 +962,13 @@ class Storage:
 
     @timed
     @db_transaction()
-    def snapshot_get_random(self, db=None, cur=None) -> Sha1Git:
+    def snapshot_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.snapshot_get_random(cur)
 
     @timed
     @db_transaction()
     def origin_visit_add(
-        self, visits: List[OriginVisit], db=None, cur=None
+        self, visits: List[OriginVisit], *, db: Db, cur=None
     ) -> Iterable[OriginVisit]:
         for visit in visits:
             origin = self.origin_get([visit.origin], db=db, cur=cur)[0]
@@ -929,15 +1010,13 @@ class Storage:
         """Add an origin visit status"""
         self.journal_writer.origin_visit_status_add([visit_status])
         db.origin_visit_status_add(visit_status, cur=cur)
-        send_metric(
-            "origin_visit_status:add", count=1, method_name="origin_visit_status"
-        )
 
     @timed
+    @process_metrics
     @db_transaction()
     def origin_visit_status_add(
-        self, visit_statuses: List[OriginVisitStatus], db=None, cur=None,
-    ) -> None:
+        self, visit_statuses: List[OriginVisitStatus], *, db: Db, cur=None,
+    ) -> Dict[str, int]:
         visit_statuses_ = []
 
         # First round to check existence (fail early if any is ko)
@@ -964,6 +1043,7 @@ class Storage:
 
         for visit_status in visit_statuses_:
             self._origin_visit_status_add(visit_status, db, cur)
+        return {"origin_visit_status:add": len(visit_statuses_)}
 
     @timed
     @db_transaction()
@@ -973,7 +1053,8 @@ class Storage:
         visit: int,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> Optional[OriginVisitStatus]:
         if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
@@ -997,7 +1078,8 @@ class Storage:
         page_token: Optional[str] = None,
         order: ListOrder = ListOrder.ASC,
         limit: int = 10,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> PagedResult[OriginVisit]:
         page_token = page_token or "0"
@@ -1034,7 +1116,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def origin_visit_find_by_date(
-        self, origin: str, visit_date: datetime.datetime, db=None, cur=None
+        self, origin: str, visit_date: datetime.datetime, *, db: Db, cur=None
     ) -> Optional[OriginVisit]:
         row_d = db.origin_visit_find_by_date(origin, visit_date, cur=cur)
         if not row_d:
@@ -1049,7 +1131,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def origin_visit_get_by(
-        self, origin: str, visit: int, db=None, cur=None
+        self, origin: str, visit: int, *, db: Db, cur=None
     ) -> Optional[OriginVisit]:
         row = db.origin_visit_get(origin, visit, cur)
         if row:
@@ -1070,7 +1152,8 @@ class Storage:
         type: Optional[str] = None,
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> Optional[OriginVisit]:
         if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
@@ -1106,7 +1189,8 @@ class Storage:
         page_token: Optional[str] = None,
         order: ListOrder = ListOrder.ASC,
         limit: int = 10,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> PagedResult[OriginVisitStatus]:
         next_page_token = None
@@ -1133,7 +1217,7 @@ class Storage:
     @timed
     @db_transaction()
     def origin_visit_status_get_random(
-        self, type: str, db=None, cur=None
+        self, type: str, *, db: Db, cur=None
     ) -> Optional[OriginVisitStatus]:
         row = db.origin_visit_get_random(type, cur)
         if row is not None:
@@ -1144,7 +1228,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=2000)
     def object_find_by_sha1_git(
-        self, ids: List[Sha1Git], db=None, cur=None
+        self, ids: List[Sha1Git], *, db: Db, cur=None
     ) -> Dict[Sha1Git, List[Dict]]:
         ret: Dict[Sha1Git, List[Dict]] = {id: [] for id in ids}
 
@@ -1159,7 +1243,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def origin_get(
-        self, origins: List[str], db=None, cur=None
+        self, origins: List[str], *, db: Db, cur=None
     ) -> Iterable[Optional[Origin]]:
         rows = db.origin_get_by_url(origins, cur)
         result: List[Optional[Origin]] = []
@@ -1172,7 +1256,7 @@ class Storage:
     @timed
     @db_transaction(statement_timeout=500)
     def origin_get_by_sha1(
-        self, sha1s: List[bytes], db=None, cur=None
+        self, sha1s: List[bytes], *, db: Db, cur=None
     ) -> List[Optional[Dict[str, Any]]]:
         return [
             dict(zip(db.origin_cols, row)) if row[0] else None
@@ -1181,14 +1265,14 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def origin_get_range(self, origin_from=1, origin_count=100, db=None, cur=None):
+    def origin_get_range(self, origin_from=1, origin_count=100, *, db: Db, cur=None):
         for origin in db.origin_get_range(origin_from, origin_count, cur):
             yield dict(zip(db.origin_get_range_cols, origin))
 
     @timed
     @db_transaction()
     def origin_list(
-        self, page_token: Optional[str] = None, limit: int = 100, *, db=None, cur=None
+        self, page_token: Optional[str] = None, limit: int = 100, *, db: Db, cur=None
     ) -> PagedResult[Origin]:
         page_token = page_token or "0"
         if not isinstance(page_token, str):
@@ -1222,7 +1306,8 @@ class Storage:
         regexp: bool = False,
         with_visit: bool = False,
         visit_types: Optional[List[str]] = None,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> PagedResult[Origin]:
         next_page_token = None
@@ -1253,7 +1338,8 @@ class Storage:
         url_pattern: str,
         regexp: bool = False,
         with_visit: bool = False,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> int:
         return db.origin_count(url_pattern, regexp, with_visit, cur)
@@ -1261,7 +1347,7 @@ class Storage:
     @timed
     @process_metrics
     @db_transaction()
-    def origin_add(self, origins: List[Origin], db=None, cur=None) -> Dict[str, int]:
+    def origin_add(self, origins: List[Origin], *, db: Db, cur=None) -> Dict[str, int]:
         urls = [o.url for o in origins]
         known_origins = set(url for (url,) in db.origin_get_by_url(urls, cur))
         # keep only one occurrence of each given origin while keeping the list
@@ -1276,11 +1362,11 @@ class Storage:
         return {"origin:add": added}
 
     @db_transaction(statement_timeout=500)
-    def stat_counters(self, db=None, cur=None):
+    def stat_counters(self, *, db: Db, cur=None):
         return {k: v for (k, v) in db.stat_counters()}
 
     @db_transaction()
-    def refresh_stat_counters(self, db=None, cur=None):
+    def refresh_stat_counters(self, *, db: Db, cur=None):
         keys = [
             "content",
             "directory",
@@ -1300,10 +1386,12 @@ class Storage:
         for key in keys:
             cur.execute("select * from swh_update_counter(%s)", (key,))
 
+    @timed
+    @process_metrics
     @db_transaction()
     def raw_extrinsic_metadata_add(
         self, metadata: List[RawExtrinsicMetadata], db, cur,
-    ) -> None:
+    ) -> Dict[str, int]:
         metadata = list(metadata)
         self.journal_writer.raw_extrinsic_metadata_add(metadata)
         counter = Counter[ExtendedObjectType]()
@@ -1331,12 +1419,9 @@ class Storage:
             )
             counter[metadata_entry.target.object_type] += 1
 
-        for (type, count) in counter.items():
-            send_metric(
-                f"{type.value}_metadata:add",
-                count=count,
-                method_name=f"{type.name.lower()}_metadata_add",
-            )
+        return {
+            f"{type.value}_metadata:add": count for (type, count) in counter.items()
+        }
 
     @db_transaction()
     def raw_extrinsic_metadata_get(
@@ -1346,7 +1431,8 @@ class Storage:
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-        db=None,
+        *,
+        db: Db,
         cur=None,
     ) -> PagedResult[RawExtrinsicMetadata]:
         if page_token:
@@ -1364,7 +1450,7 @@ class Storage:
             return PagedResult(next_page_token=None, results=[],)
 
         rows = db.raw_extrinsic_metadata_get(
-            type, str(target), authority_id, after_time, after_fetcher, limit + 1, cur,
+            str(target), authority_id, after_time, after_fetcher, limit + 1, cur,
         )
         rows = [dict(zip(db.raw_extrinsic_metadata_get_cols, row)) for row in rows]
         results = []
@@ -1389,29 +1475,49 @@ class Storage:
 
         return PagedResult(next_page_token=next_page_token, results=results,)
 
+    @db_transaction()
+    def raw_extrinsic_metadata_get_by_ids(
+        self, ids: List[Sha1Git], *, db: Db, cur=None,
+    ) -> List[RawExtrinsicMetadata]:
+        return [
+            converters.db_to_raw_extrinsic_metadata(
+                dict(zip(db.raw_extrinsic_metadata_get_cols, row))
+            )
+            for row in db.raw_extrinsic_metadata_get_by_ids(ids)
+        ]
+
+    @db_transaction()
+    def raw_extrinsic_metadata_get_authorities(
+        self, target: ExtendedSWHID, *, db: Db, cur=None,
+    ) -> List[MetadataAuthority]:
+        return [
+            MetadataAuthority(
+                type=MetadataAuthorityType(authority_type), url=authority_url
+            )
+            for (
+                authority_type,
+                authority_url,
+            ) in db.raw_extrinsic_metadata_get_authorities(str(target), cur)
+        ]
+
     @timed
+    @process_metrics
     @db_transaction()
     def metadata_fetcher_add(
-        self, fetchers: List[MetadataFetcher], db=None, cur=None
-    ) -> None:
+        self, fetchers: List[MetadataFetcher], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         fetchers = list(fetchers)
         self.journal_writer.metadata_fetcher_add(fetchers)
         count = 0
         for fetcher in fetchers:
-            if fetcher.metadata is None:
-                raise StorageArgumentException(
-                    "MetadataFetcher.metadata may not be None in metadata_fetcher_add."
-                )
-            db.metadata_fetcher_add(
-                fetcher.name, fetcher.version, dict(fetcher.metadata), cur=cur
-            )
+            db.metadata_fetcher_add(fetcher.name, fetcher.version, cur=cur)
             count += 1
-        send_metric("metadata_fetcher:add", count=count, method_name="metadata_fetcher")
+        return {"metadata_fetcher:add": count}
 
     @timed
     @db_transaction(statement_timeout=500)
     def metadata_fetcher_get(
-        self, name: str, version: str, db=None, cur=None
+        self, name: str, version: str, *, db: Db, cur=None
     ) -> Optional[MetadataFetcher]:
         row = db.metadata_fetcher_get(name, version, cur=cur)
         if not row:
@@ -1419,31 +1525,23 @@ class Storage:
         return MetadataFetcher.from_dict(dict(zip(db.metadata_fetcher_cols, row)))
 
     @timed
+    @process_metrics
     @db_transaction()
     def metadata_authority_add(
-        self, authorities: List[MetadataAuthority], db=None, cur=None
-    ) -> None:
+        self, authorities: List[MetadataAuthority], *, db: Db, cur=None
+    ) -> Dict[str, int]:
         authorities = list(authorities)
         self.journal_writer.metadata_authority_add(authorities)
         count = 0
         for authority in authorities:
-            if authority.metadata is None:
-                raise StorageArgumentException(
-                    "MetadataAuthority.metadata may not be None in "
-                    "metadata_authority_add."
-                )
-            db.metadata_authority_add(
-                authority.type.value, authority.url, dict(authority.metadata), cur=cur
-            )
+            db.metadata_authority_add(authority.type.value, authority.url, cur=cur)
             count += 1
-        send_metric(
-            "metadata_authority:add", count=count, method_name="metadata_authority"
-        )
+        return {"metadata_authority:add": count}
 
     @timed
     @db_transaction()
     def metadata_authority_get(
-        self, type: MetadataAuthorityType, url: str, db=None, cur=None
+        self, type: MetadataAuthorityType, url: str, *, db: Db, cur=None
     ) -> Optional[MetadataAuthority]:
         row = db.metadata_authority_get(type.value, url, cur=cur)
         if not row:
