@@ -17,7 +17,7 @@ comment on column dbversion.description is 'Release description';
 
 -- latest schema version
 insert into dbversion(version, release, description)
-      values(173, now(), 'Work In Progress');
+      values(177, now(), 'Work In Progress');
 
 -- a SHA1 checksum
 create domain sha1 as bytea check (length(value) = 20);
@@ -401,30 +401,26 @@ create table metadata_fetcher
 (
   id            serial  not null,
   name          text    not null,
-  version       text    not null,
-  metadata      jsonb   not null
+  version       text    not null
 );
 
 comment on table metadata_fetcher is 'Tools used to retrieve metadata';
 comment on column metadata_fetcher.id is 'Internal identifier of the fetcher';
 comment on column metadata_fetcher.name is 'Fetcher name';
 comment on column metadata_fetcher.version is 'Fetcher version';
-comment on column metadata_fetcher.metadata is 'Extra information about the fetcher';
 
 
 create table metadata_authority
 (
   id            serial  not null,
   type          text    not null,
-  url           text    not null,
-  metadata      jsonb   not null
+  url           text    not null
 );
 
 comment on table metadata_authority is 'Metadata authority information';
 comment on column metadata_authority.id is 'Internal identifier of the authority';
 comment on column metadata_authority.type is 'Type of authority (deposit_client/forge/registry)';
 comment on column metadata_authority.url is 'Authority''s uri';
-comment on column metadata_authority.metadata is 'Other metadata about authority';
 
 
 -- Extrinsic metadata on a DAG objects and origins.
@@ -509,7 +505,8 @@ create table extid
   extid_type  text not null,
   extid       bytea not null,
   target_type object_type not null,
-  target      sha1_git not null
+  target      sha1_git not null,
+  extid_version bigint not null default 0
 );
 
 comment on table extid is 'Correspondance SWH object (SWHID) <-> original revision id (vcs id)';
@@ -517,3 +514,4 @@ comment on column extid.extid_type is 'ExtID type';
 comment on column extid.extid is 'Intrinsic identifier of the object (e.g. hg revision)';
 comment on column extid.target_type is 'Type of SWHID of the referenced SWH object';
 comment on column extid.target is 'Value (hash) of SWHID of the refenced SWH object';
+comment on column extid.extid_version is 'Version of the extid type for the given original object';
