@@ -1,4 +1,4 @@
-# Copyright (C) 2019 The Software Heritage developers
+# Copyright (C) 2019-2021 The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -10,11 +10,11 @@ import pytest
 from swh.core.pytest_plugin import RPCTestAdapter
 from swh.storage import get_storage
 from swh.storage.api import client, server
-from swh.storage.buffer import BufferingProxyStorage
-from swh.storage.filter import FilteringProxyStorage
 from swh.storage.in_memory import InMemoryStorage
 from swh.storage.postgresql.storage import Storage as DbStorage
-from swh.storage.retry import RetryingProxyStorage
+from swh.storage.proxies.buffer import BufferingProxyStorage
+from swh.storage.proxies.filter import FilteringProxyStorage
+from swh.storage.proxies.retry import RetryingProxyStorage
 
 STORAGES = [
     pytest.param(cls, real_class, kwargs, id=cls)
@@ -22,9 +22,9 @@ STORAGES = [
         ("remote", client.RemoteStorage, {"url": "url"}),
         ("memory", InMemoryStorage, {}),
         (
-            "local",
+            "postgresql",
             DbStorage,
-            {"db": "postgresql://db", "objstorage": {"cls": "memory", "args": {}}},
+            {"db": "postgresql://db", "objstorage": {"cls": "memory"}},
         ),
         ("filter", FilteringProxyStorage, {"storage": {"cls": "memory"}}),
         ("buffer", BufferingProxyStorage, {"storage": {"cls": "memory"}}),
@@ -64,7 +64,7 @@ def test_get_storage_failure():
 
     """
     with pytest.raises(ValueError, match="Unknown storage class `unknown`"):
-        get_storage("unknown", args=[])
+        get_storage("unknown")
 
 
 def test_get_storage_pipeline():
@@ -106,7 +106,8 @@ def test_get_storage_pipeline_legacy_args():
 
 # the "remote" and "pipeline" cases are tested in dedicated test functions below
 @pytest.mark.parametrize(
-    "cls,real_class,kwargs", [x for x in STORAGES if x.id not in ("remote", "local")]
+    "cls,real_class,kwargs",
+    [x for x in STORAGES if x.id not in ("remote", "local", "postgresql")],
 )
 def test_get_storage_check_config(cls, real_class, kwargs, monkeypatch):
     """Instantiating an existing storage with check_config should be ok
@@ -116,18 +117,15 @@ def test_get_storage_check_config(cls, real_class, kwargs, monkeypatch):
 
 
 @patch("swh.storage.postgresql.storage.psycopg2.pool")
-def test_get_storage_local_check_config(mock_pool, monkeypatch):
+@pytest.mark.parametrize("clazz", ["local", "postgresql"])
+def test_get_storage_local_check_config(mock_pool, monkeypatch, clazz):
     """Instantiating a local storage with check_config should be ok
 
     """
     mock_pool.ThreadedConnectionPool.return_value = None
     check_backend_check_config(
         monkeypatch,
-        {
-            "cls": "local",
-            "db": "postgresql://db",
-            "objstorage": {"cls": "memory", "args": {}},
-        },
+        {"cls": clazz, "db": "postgresql://db", "objstorage": {"cls": "memory"}},
         backend_storage_cls=DbStorage,
     )
 
